@@ -6,7 +6,7 @@ import re, logging, weakref, time, threading, abc
 from datetime import datetime
 
 from .serial_comms import SerialComms
-from .exceptions import CommandError, InvalidStateException, CmeError, CmsError, InterruptedException, TimeoutException
+from .exceptions import CommandError, InvalidStateException, CmeError, CmsError, InterruptedException, TimeoutException, PinRequiredError, IncorrectPinError, SmscNumberUnknownError
 from .pdu import encodeSmsSubmitPdu, decodeSmsPdu
 from .util import SimpleOffsetTzInfo
 
@@ -26,37 +26,52 @@ class GsmModem(SerialComms):
     # Used for parsing new SMS message indications
     CMTI_REGEX = re.compile(r'^\+CMTI:\s*([^,]+),(\d+)$')
     # Used for parsing SMS message reads (text mode)
-    CMGR_SM_DELIVER_REGEX_TEXT = re.compile(r'^\+CMGR: "([^"]+)","([^"]+)",[^,]*,"([^"]+)"$')
+    CMGR_SM_DELIVER_REGEX_TEXT = None
+    # Used for parsing SMS status report message reads (text mode)
+    CMGR_SM_REPORT_REGEXT_TEXT = None
     # Used for parsing SMS message reads (PDU mode)
-    CMGR_SM_DELIVER_REGEX_PDU = re.compile(r'^\+CMGR: (\d+),(\d*),(\d+)$')
+    CMGR_REGEX_PDU = None
     # Used for parsing USSD event notifications
-    CUSD_REGEX = re.compile(r'^\+CUSD:\s*(\d),"(.*)",(\d+)$', re.DOTALL)    
+    CUSD_REGEX = re.compile(r'^\+CUSD:\s*(\d),"(.*)",(\d+)$', re.DOTALL)
+    # Used for parsing SMS status reports
+    CDSI_REGEX = re.compile(r'\+CDSI:\s*"([^,]+)",(\d+)$')
     
-    def __init__(self, port, baudrate=115200, incomingCallCallbackFunc=None, smsReceivedCallbackFunc=None):
+    def __init__(self, port, baudrate=115200, incomingCallCallbackFunc=None, smsReceivedCallbackFunc=None, smsStatusReportCallback=None):
         super(GsmModem, self).__init__(port, baudrate, notifyCallbackFunc=self._handleModemNotification)
         self.incomingCallCallback = incomingCallCallbackFunc or self._placeholderCallback
         self.smsReceivedCallback = smsReceivedCallbackFunc or self._placeholderCallback
+        self.smsStatusReportCallback = smsStatusReportCallback or self._placeholderCallback
         # Flag indicating whether caller ID for incoming call notification has been set up
         self._callingLineIdentification = False
         # Flag indicating whether incoming call notifications have extended information
         self._extendedIncomingCallIndication = False
         # Current active calls (ringing and/or answered), key is the unique call ID (not the remote number)
         self.activeCalls = {}
-        # Dict containing sent SMS messages (to track their delivery status)
-        self.sentSms = {}
+        # Dict containing sent SMS messages (for auto-tracking their delivery status)
+        self.sentSms = weakref.WeakValueDictionary()
         self._ussdSessionEvent = None # threading.Event
         self._ussdResponse = None # gsmmodem.modem.Ussd
+        self._smsStatusReportEvent = None # threading.Event
         self._dialEvent = None # threading.Event
         self._dialResponse = None # gsmmodem.modem.Call
         self._waitForAtdResponse = True # Flag that controls if we should wait for an immediate response to ATD, or not
         self.callStatusUpdates = [] # populated during connect() - contains regexes and handlers for detecting/handling call status updates
         self._writeWait = 0 # Time (in seconds to wait after writing a command (adjusted when 515 errors are detected)
         self._smsTextMode = False # Storage variable for the smsTextMode property
+        self._smscNumber = None # Default SMSC number
+        self._smsRef = 0 # Sent SMS reference counter
+
+    def connect(self, pin=None):
+        """ Opens the port and initializes the modem and SIM card
+         
+        @param pin: The SIM card PIN code, if any
+        @type pin: str
         
-    def connect(self, runInit=True):
-        """ Opens the port and initializes the modem """
+        @raise PinRequiredError: if the SIM card requires a PIN but none was provided
+        @raise IncorrectPinError: if the specified PIN is incorrect
+        """
         self.log.info('Connecting to modem on port %s at %dbps', self.port, self.baudrate)        
-        super(GsmModem, self).connect()                
+        super(GsmModem, self).connect()
         # Send some initialization commands to the modem        
         self.write('ATZ') # reset configuration
         self.write('ATE0') # echo off
@@ -68,7 +83,14 @@ class GsmModem(SerialComms):
             pass # just ignore if the +CFUN command isn't supported
          
         self.write('AT+CMEE=1') # enable detailed error messages
-        
+
+        # Unlock the SIM card if needed
+        if self.write('AT+CPIN?')[0] != '+CPIN: READY':
+            if pin != None:
+                self.write('AT+CPIN="{0}"'.format(pin))
+            else:
+                raise PinRequiredError('AT+CPIN')
+
         # Get list of supported commands from modem
         commands = self.supportedCommands
 
@@ -110,6 +132,9 @@ class GsmModem(SerialComms):
                 
         # SMS setup
         self.write('AT+CMGF={0}'.format(1 if self._smsTextMode else 0)) # Switch to text or PDU mode for SMS messages
+        self._compileSmsRegexes()
+        if self._smscNumber != None:
+            self.write('AT+CSCA="{0}"'.format(self._smscNumber)) # Set default SMSC number
         self.write('AT+CSMP=49,167,0,0') # Enable delivery reports
         
         # Set message storage, but first check what the modem supports - example response: +CPMS: (("SM","BM","SR"),("SM"))
@@ -121,10 +146,10 @@ class GsmModem(SerialComms):
             if cpmsItems[i] not in cpmsSupport[i]:
                 cpmsItems[i] = ''
         self.write('AT+CPMS={0}'.format(','.join(cpmsItems))) # Set message storage        
-        #self.write('AT+CPMS="SM","SM","SR"') # Set message storage
+        
         self.write('AT+CNMI=2,1,0,2') # Set message notifications
         
-        # Incoming call notification setup        
+        # Incoming call notification setup
         try:
             self.write('AT+CLIP=1') # Enable calling line identification presentation
         except CommandError, clipError:
@@ -170,15 +195,19 @@ class GsmModem(SerialComms):
                 if cmErrorMatch:
                     errorType = cmErrorMatch.group(1)
                     errorCode = int(cmErrorMatch.group(2))
-                    if errorCode == 515:
+                    if errorCode == 515 or errorCode == 14:
                         # 515 means: "Please wait, init or command processing in progress."
+                        # 14 means "SIM busy"
                         self._writeWait += 0.2 # Increase waiting period temporarily
                         # Retry the command after waiting a bit
                         self.log.debug('515 error detected; self._writeWait adjusted to %ds', self._writeWait)
                         time.sleep(self._writeWait)
                         result = self.write(data, waitForResponse, timeout, parseError, writeTerm, expectedResponseTermSeq)
                         self.log.debug('self_writeWait set to 0.1 because of recovering from 515 error')
-                        self._writeWait = 0.1 # Set this to something sane for further commands
+                        if errorCode == 515:
+                            self._writeWait = 0.1 # Set this to something sane for further commands
+                        else:
+                            self._writeWait = 0 # The modem was just waiting for the SIM card
                         return result
                     if errorType == 'CME':
                         raise CmeError(data, int(errorCode))
@@ -255,6 +284,38 @@ class GsmModem(SerialComms):
             if self.alive:
                 self.write('AT+CMGF={0}'.format(1 if textMode else 0))
             self._smsTextMode = textMode
+            self._compileSmsRegexes()
+            
+    def _compileSmsRegexes(self):
+        """ Compiles regular expression used for parsing SMS messages based on current mode """
+        if self._smsTextMode:
+            if self.CMGR_SM_DELIVER_REGEX_TEXT == None:
+                self.CMGR_SM_DELIVER_REGEX_TEXT = re.compile(r'^\+CMGR: "([^"]+)","([^"]+)",[^,]*,"([^"]+)"$')
+                self.CMGR_SM_REPORT_REGEXT_TEXT = re.compile(r'^\+CMGR: "([^"]+)",\d+,(\d+),"{0,1}([^"]*)"{0,1},\d*,"([^"]+)","([^"]+)",(\d+)$')
+        elif self.CMGR_REGEX_PDU == None:
+            self.CMGR_REGEX_PDU = re.compile(r'^\+CMGR: (\d+),(\d*),(\d+)$')
+            
+    @property
+    def smsc(self):
+        """ @return: The default SMSC number stored on the SIM card """
+        if self._smscNumber == None:
+            try:
+                readSmsc = self.write('AT+CSCA?')
+            except SmscNumberUnknownError:
+                pass # Some modems return a CMS 330 error if the value isn't set
+            else:
+                if len(readSmsc) == 2:
+                    cscaMatch = re.match(r'\+CSCA:\s*"([^,]+)",(\d+)$', readSmsc[0])
+                    if cscaMatch:
+                        self._smscNumber = cscaMatch.group(1)
+        return self._smscNumber
+    @smsc.setter
+    def smsc(self, smscNumber):
+        """ Set the default SMSC number to use when sending SMS messages """
+        if smscNumber != self._smscNumber:
+            if self.alive:
+                self.write('AT+CSCA="{0}"'.format(smscNumber))
+            self._smscNumber = smscNumber
 
     def waitForNetworkCoverage(self, timeout=None):
         """ Block until the modem has GSM network coverage.
@@ -291,18 +352,31 @@ class GsmModem(SerialComms):
         
         @param destination: The recipient's phone number
         @param text: The message text
-        """
-        sms = SentSms(destination, text)
+        """        
         if self._smsTextMode:
             self.write('AT+CMGS="{0}"'.format(destination), timeout=3, expectedResponseTermSeq='> ')
-            self.write(text, timeout=15, writeTerm=chr(26))
+            result = self.write(text, timeout=15, writeTerm=chr(26))[0]
         else:
-            smsPdu, tpduLength = encodeSmsSubmitPdu(destination, text)
+            smsPdu, tpduLength = encodeSmsSubmitPdu(destination, text, reference=self._smsRef)
             smsPduHex = str(smsPdu).encode('hex').upper()
             self.write('AT+CMGS={0}'.format(tpduLength), timeout=3, expectedResponseTermSeq='> ')
-            self.write(smsPduHex, timeout=15, writeTerm=chr(26))
+            result = self.write(smsPduHex, timeout=15, writeTerm=chr(26))[0] # example: +CMGS: xx
+        reference = int(result[7:])
+        self._smsRef = reference + 1
+        if self._smsRef > 255:
+            self._smsRef = 0
+        sms = SentSms(destination, text, reference)
+        # Add a weak-referenced entry for this SMS (allows us to update the SMS state if a status report is received)
+        self.sentSms[reference] = sms
+        if waitForDeliveryReport:
+            self._smsStatusReportEvent = threading.Event()
+            if self._smsStatusReportEvent.wait(deliveryTimeout):
+                self._smsStatusReportEvent = None
+            else: # Response timed out
+                self._smsStatusReportEvent = None
+                raise TimeoutException()
         return sms
-    
+
     def sendUssd(self, ussdString, responseTimeout=15):
         """ Starts a USSD session by dialing the the specified USSD string, or \
         sends the specified string in the existing USSD session (if any)
@@ -355,11 +429,10 @@ class GsmModem(SerialComms):
         threading.Thread(target=self.__threadedHandleModemNotification, kwargs={'lines': lines}).start()
     
     def __threadedHandleModemNotification(self, lines):
-        """ Implementation of _handleModemNotification() to be run in a separate thread 
+        """ Implementation of _handleModemNotification() to be run in a separate thread
         
         @param lines The lines that were read
-        """        
-        #firstLine = lines[0]
+        """
         for line in lines:
             if 'RING' in line:
                 # Incoming call (or existing call is ringing)
@@ -373,6 +446,10 @@ class GsmModem(SerialComms):
                 # USSD notification - either a response or a MT-USSD ("push USSD") message
                 self._handleUssd(line)
                 return
+            elif line.startswith('+CDSI'):
+                # SMS status report
+                self._handleSmsStatusReport(line)
+                return
             else:
                 # Check for call status updates            
                 for updateRegex, handlerFunc in self.callStatusUpdates:
@@ -385,6 +462,7 @@ class GsmModem(SerialComms):
         self.log.debug('Unhandled unsolicited modem notification: %s', lines)    
     
     def _handleIncomingCall(self, lines):
+        self.log.debug('Handling incoming call')
         ringLine = lines.pop(0)
         if self._extendedIncomingCallIndication:
             callType = ringLine.split(' ', 1)[1]
@@ -448,35 +526,73 @@ class GsmModem(SerialComms):
     
     def _handleSmsReceived(self, notificationLine):
         """ Handler for "new SMS" unsolicited notification line """
+        self.log.debug('SMS message received')
         cmtiMatch = self.CMTI_REGEX.match(notificationLine)
         if cmtiMatch:
             msgIndex = cmtiMatch.group(2)
             sms = self._readStoredSmsMessage(msgIndex)
             self._deleteStoredMessage(msgIndex)
-            self.smsReceivedCallback(sms)            
+            self.smsReceivedCallback(sms)
+    
+    def _handleSmsStatusReport(self, notificationLine):
+        """ Handler for SMS status reports """
+        self.log.debug('SMS status report received')
+        cdsiMatch = self.CDSI_REGEX.match(notificationLine)
+        if cdsiMatch:
+            msgIndex = cdsiMatch.group(2)
+            report = self._readStoredSmsMessage(msgIndex)
+            self._deleteStoredMessage(msgIndex)
+            # Update sent SMS status if possible            
+            if report.reference in self.sentSms:                
+                self.sentSms[report.reference].report = report
+            if self._smsStatusReportEvent:                
+                # A sendSms() call is waiting for this response - notify waiting thread
+                self._smsStatusReportEvent.set()
+            else:
+                # Nothing is waiting for this report directly - use callback
+                self.smsStatusReportCallback(report)
     
     def _readStoredSmsMessage(self, msgIndex):
         msgData = self.write('AT+CMGR={0}'.format(msgIndex))
         # Parse meta information
         if self._smsTextMode:
             cmgrMatch = self.CMGR_SM_DELIVER_REGEX_TEXT.match(msgData[0])
-            if not cmgrMatch:
-                raise CommandError('Failed to parse the SMS message +CMGR response: {0}'.format(msgData))
-            msgStatus, number, msgTime = cmgrMatch.groups()
-            msgText = '\n'.join(msgData[1:-1])
-            # Parse date/time
-            timeStr = msgTime[:-3]
-            tzOffsetHours = int(msgTime[-3:])        
-            parsedTime = datetime.strptime(timeStr, '%y/%m/%d,%H:%M:%S').replace(tzinfo=SimpleOffsetTzInfo(tzOffsetHours))
-            return ReceivedSms(self, Sms.TEXT_MODE_STATUS_MAP[msgStatus], number, parsedTime, msgText)
+            if cmgrMatch:
+                msgStatus, number, msgTime = cmgrMatch.groups()
+                msgText = '\n'.join(msgData[1:-1])
+                # Parse date/time
+                timeStr = msgTime[:-3]
+                tzOffsetHours = int(msgTime[-3:])        
+                parsedTime = datetime.strptime(timeStr, '%y/%m/%d,%H:%M:%S').replace(tzinfo=SimpleOffsetTzInfo(tzOffsetHours))
+                return ReceivedSms(self, Sms.TEXT_MODE_STATUS_MAP[msgStatus], number, parsedTime, msgText)
+            else:
+                # Try parsing status report
+                cmgrMatch = self.CMGR_SM_REPORT_REGEXT_TEXT.match(msgData[0])
+                if cmgrMatch:
+                    msgStatus, reference, number, sentTime, deliverTime, deliverStatus = cmgrMatch.groups()
+                    # Parse date/time
+                    timeStr = sentTime[:-3]
+                    tzOffsetHours = int(sentTime[-3:])
+                    parsedSentTime = datetime.strptime(timeStr, '%y/%m/%d,%H:%M:%S').replace(tzinfo=SimpleOffsetTzInfo(tzOffsetHours))
+                    timeStr = sentTime[:-3]
+                    tzOffsetHours = int(sentTime[-3:])
+                    parsedDeliveryTime = datetime.strptime(timeStr, '%y/%m/%d,%H:%M:%S').replace(tzinfo=SimpleOffsetTzInfo(tzOffsetHours))
+                    return StatusReport(self, Sms.TEXT_MODE_STATUS_MAP[msgStatus], int(reference), number, parsedSentTime, parsedDeliveryTime, int(deliverStatus))
+                else:
+                    raise CommandError('Failed to parse the SMS message +CMGR response: {0}'.format(msgData))
         else:
-            cmgrMatch = self.CMGR_SM_DELIVER_REGEX_PDU.match(msgData[0])
+            cmgrMatch = self.CMGR_REGEX_PDU.match(msgData[0])
             if not cmgrMatch:
                 raise CommandError('Failed to parse the SMS message +CMGR response: {0}'.format(msgData))
             stat, alpha, length = cmgrMatch.groups()
             pdu = msgData[1]
             smsDict = decodeSmsPdu(pdu)
-            return ReceivedSms(self, int(stat), smsDict['number'], smsDict['time'], smsDict['text'], smsDict['smsc'])
+            if smsDict['type'] == 'SMS-DELIVER':
+                return ReceivedSms(self, int(stat), smsDict['number'], smsDict['time'], smsDict['text'], smsDict['smsc'])
+            elif smsDict['type'] == 'SMS-STATUS-REPORT':
+                return StatusReport(self, int(stat), smsDict['reference'], smsDict['number'], smsDict['time'], smsDict['discharge'], smsDict['status'])
+            else:
+                raise CommandError('Invalid PDU type for _readStoredSmsMessage(): {0}'.format(smsDict['type']))
             
     def _deleteStoredMessage(self, msgIndex):
         self.write('AT+CMGD={0}'.format(msgIndex))
@@ -633,11 +749,47 @@ class SentSms(Sms):
     """ An SMS message that has been sent (MO) """
         
     ENROUTE = 0 # Status indicating message is still enroute to destination
-    RECEIVED = 1 # Status indicating message has been received by destination handset
-    
-    def __init__(self, number, text, smsc=None):
+    DELIVERED = 1 # Status indicating message has been received by destination handset
+    FAILED = 2 # Status indicating message delivery has failed
+
+    def __init__(self, number, text, reference, smsc=None):
         super(SentSms, self).__init__(number, text, smsc)
-        self.status = SentSms.ENROUTE
+        self.report = None # Status report for this SMS (StatusReport object)
+        self.reference = reference
+        
+    @property
+    def status(self):
+        """ Status of this SMS. Can be ENROUTE, DELIVERED or FAILED
+        
+        The actual status report object may be accessed via the 'report' attribute
+        if status is 'DELIVERED' or 'FAILED'
+        """
+        if self.report == None:
+            return SentSms.ENROUTE
+        else:
+            return SentSms.DELIVERED if self.report.deliveryStatus == StatusReport.DELIVERED else SentSms.FAILED
+
+
+class StatusReport(Sms):
+    """ An SMS status/delivery report 
+    
+    Note: the 'status' attribute of this class refers to this status report SM's status (whether
+    it has been read, etc). To find the status of the message that caused this status report,
+    use the 'deliveryStatus' attribute.
+    """
+        
+    DELIVERED = 0 # SMS delivery status: delivery successful    
+    FAILED = 68 # SMS delivery status: delivery failed
+    
+    def __init__(self, gsmModem, status, reference, number, timeSent, timeFinalized, deliveryStatus, smsc=None):
+        super(StatusReport, self).__init__(number, None, smsc)
+        self._gsmModem = weakref.proxy(gsmModem)
+        self.status = status
+        self.reference = reference
+        self.timeSent = timeSent
+        self.timeFinalized = timeFinalized
+        self.deliveryStatus = deliveryStatus
+        
 
 class Ussd(object):
     """ Unstructured Supplementary Service Data (USSD) message.
